<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Prophecy\Argument;
use PhpSpec\ObjectBehavior;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\NewsML\Parsers\ImageGalleryParser;

class ImageGalleryParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ImageGalleryParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <object id="im-imagegallery-121d0596d7c0161056159b69be99092c" type="x-im/imagegallery" uuid="1a1cba5a-7d79-4642-bef9-51012db143ed">
                <data>
                    <text>The gallery!</text>
                </data>
                <links>
                    <link rel="image" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                        <data>
                            <text>Weird line</text>
                            <height>169</height>
                            <width>300</width>
                        </data>
                    </link>
                    <link rel="image" type="x-im/image" uri="im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png" uuid="d027b036-0b84-5287-b37a-901f6f8f54b4">
                        <data>
                            <text>Flag</text>
                            <height>960</height>
                            <width>1920</width>
                        </data>
                    </link>
                </links>
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveValuesKeys(
                [
                    "id" => "im-imagegallery-121d0596d7c0161056159b69be99092c",
                    "type" => "x-im/imagegallery",
                    "uuid" => "1a1cba5a-7d79-4642-bef9-51012db143ed",
                    "images" => [
                        [
                            "rel" => "image",
                            "type" => "x-im/image",
                            "uri" => "im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg",
                            "uuid" => "d83dad15-9476-54c1-a57d-495757664dae",
                            "width" => "300",
                            "height" => "169",
                            "text" => "Weird line",
                            "alttext" => "",
                        ],
                        [
                            "rel" => "image",
                            "type" => "x-im/image",
                            "uri" => "im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png",
                            "uuid" => "d027b036-0b84-5287-b37a-901f6f8f54b4",
                            "width" => "1920",
                            "height" => "960",
                            "text" => "Flag",
                            "alttext" => "",
                        ],
                    ],
                    "caption" => "The gallery!",
                ]
            );
    }
    public function it_should_have_default_values()
    {
        $xml = '
            <object id="im-imagegallery-121d0596d7c0161056159b69be99092c" type="x-im/imagegallery" uuid="1a1cba5a-7d79-4642-bef9-51012db143ed">
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveValuesKeys([
                "id" => "im-imagegallery-121d0596d7c0161056159b69be99092c",
                "type" => "x-im/imagegallery",
                "uuid" => "1a1cba5a-7d79-4642-bef9-51012db143ed",
                "images" => [],
                "caption" => "",
            ]);
    }
    public function it_should_have_class_types()
    {
        $xml = '
            <object id="im-imagegallery-121d0596d7c0161056159b69be99092c" type="x-im/imagegallery" uuid="1a1cba5a-7d79-4642-bef9-51012db143ed">
                <links>
                    <link rel="image" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                        <data>
                            <text>Weird line</text>
                            <height>169</height>
                            <width>300</width>
                        </data>
                    </link>
                </links>
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveTypes();
    }
    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/imagegallery');
    }
    public function getMatchers(): array
    {
        return [
            'haveTypes' => function ($subject) {
                return $subject[0] instanceof Item && $subject[0]->images[0] instanceof ImageItem;
            },
            'haveValuesKeys' => function ($subject, $values) {
                $item = $subject[0]->getProperties();
                $images = [];
                foreach ($item['images'] as $image) {
                    $images[] = $image->getProperties();
                }

                $item['images'] = $images;
                return $item === $values;
            },
        ];
    }
}
