<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Prophecy\Argument;
use SimpleXMLIterator;
use PhpSpec\ObjectBehavior;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\NewsML\Parsers\ImageLinkParser;


/**
 * @todo Everything
 */
class ImageLinkParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ImageLinkParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                <data>
                    <width>300</width>
                    <height>169</height>
                </data>
                <data>
                    <width>3000</width>
                    <height>169</height>
                    <imageInstructions/>
                    <text/>
                    <alttext/>
                </data>
            </link>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveType();
        $this->parse($el)
            ->shouldHaveArrayKey([
                "rel" => "self",
                "type" => "x-im/image",
                "uri" => "im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg",
                "uuid" => "d83dad15-9476-54c1-a57d-495757664dae",
                "width" => "300",
                "height" => "169",
                "text" => '',
                "alttext" => '',
            ]);
    }
    public function it_should_have_default_values()
    {
        $xml = '
            <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae"> 
            </link>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                "rel" => "self",
                "type" => "x-im/image",
                "uri" => "im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg",
                "uuid" => "d83dad15-9476-54c1-a57d-495757664dae",
            ]);
    }
    public function it_should_have_text_markup()
    {
        $xml = '
            <link rel="self" type="x-im/image" uri="im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png" uuid="d027b036-0b84-5287-b37a-901f6f8f54b4">
                <data>
                    <width>1920</width>
                    <height>960</height>
                    <imageInstructions/>
                    <text>
                        <mark>hey</mark>
                        <em>there</em>
                        <strong>general</strong>
                        <a href="google.com">kenobi</a>
                        !
                    </text>
                    <alttext>hello</alttext>
                </data>
            </link>
            ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                "rel" => "self",
                "type" => "x-im/image",
                "uri" => "im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png",
                "uuid" => "d027b036-0b84-5287-b37a-901f6f8f54b4",
                "width" => "1920",
                "height" => "960",
                "text" => '<mark>hey</mark><em>there</em><strong>general</strong><a href="google.com">kenobi</a>!',
                "alttext" => 'hello',
            ]);
    }
    public function it_should_have_authors()
    {
        $xml = '
            <link rel="self" type="x-im/image" uri="im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png" uuid="d027b036-0b84-5287-b37a-901f6f8f54b4">
                <data>
                </data>
                <links>
                    <link rel="author" uuid="5884e555-468b-41cd-a14b-d1768eb8ccf0" title="Jonathan Nilsson" type="x-im/author">
                        <data>
                            <email>infomaker@info.com</email>
                        </data>
                    </link>
                    <link rel="author" uuid="2ca22e98-c418-4330-ae00-4fa2127cae2d" title="Andy Cederlund" type="x-im/author">
                        <data>
                            <email>andy.cederlund@navigaglobal.com</email>
                        </data>
                    </link>
                </links>
            </link>
            ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                "rel" => "self",
                "type" => "x-im/image",
                "uri" => "im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png",
                "uuid" => "d027b036-0b84-5287-b37a-901f6f8f54b4",
                "width" => "0",
                "height" => "0",
                "text" => '',
                "alttext" => '',
                "authors" => [
                    [
                        "rel" => "author",
                        "uuid" => "5884e555-468b-41cd-a14b-d1768eb8ccf0",
                        "title" => "Jonathan Nilsson",
                        "type" => "x-im/author",
                    ],
                    [
                        "rel" => "author",
                        "uuid" => "2ca22e98-c418-4330-ae00-4fa2127cae2d",
                        "title" => "Andy Cederlund",
                        "type" => "x-im/author",
                    ]
                ]
            ]);
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayKey' => function ($subject, $values) {
                return $subject[0]->getProperties() === $values;
            },
            'haveType' => function ($subject) {
                return $subject[0] instanceof ImageItem;
            },
        ];
    }
}
