<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\SocialEmbedParser;
use PhpSpec\ObjectBehavior;
use Infomaker\Everyware\NewsML\Item;
use Prophecy\Argument;

class SocialEmbedParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(SocialEmbedParser::class);
    }

    public function it_should_parse_instagram_objects()
    {
        $xml = <<<XML
<object id="socialembed-39860e0d53babce67db7f8a49daf4056" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/instagram" url="https://www.instagram.com/p/zvV3xmOvb3/" uri="im://instagram/undefined"/>
        <link rel="alternate" type="text/html" url="https://www.instagram.com/p/zvV3xmOvb3/" title="infomakernews postade en bild på Instagram. ">
            <data>
                <context>Social</context>
                <provider>Instagram</provider>
            </data>
        </link>
        <link rel="alternate" type="image/jpg" url="https://scontent.cdninstagram.com/v/t51.2885-15/e15/11008312_1453406918282922_220940562_n.jpg?_nc_ht=scontent.cdninstagram.com&amp;_nc_cat=106&amp;_nc_ohc=CdLhB86ea28AX_BYyPo&amp;tp=18&amp;oh=d8e2888b5909e81fbe56daf887f066de&amp;oe=5FDBAACD">
            <data>
                <width>640</width>
                <height>640</height>
            </data>
        </link>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-39860e0d53babce67db7f8a49daf4056',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/instagram',
                    'url' => 'https://www.instagram.com/p/zvV3xmOvb3/',
                    'uri' => 'im://instagram/undefined',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://www.instagram.com/p/zvV3xmOvb3/',
                    'title' => 'infomakernews postade en bild på Instagram. ',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'image/jpg',
                    'url' => 'https://scontent.cdninstagram.com/v/t51.2885-15/e15/11008312_1453406918282922_220940562_n.jpg?_nc_ht=scontent.cdninstagram.com&_nc_cat=106&_nc_ohc=CdLhB86ea28AX_BYyPo&tp=18&oh=d8e2888b5909e81fbe56daf887f066de&oe=5FDBAACD',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_parse_facebook_post_object()
    {
        $xml = <<<XML
<object id="socialembed-f5da770b839dc92b2bcfae54c804ae97" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/facebook-post" url="https://www.facebook.com/stotenisalen/posts/10158053045049843" uri="im://facebook-post/10158053045049843"/>
        <link rel="alternate" type="text/html" url="https://www.facebook.com/stotenisalen/posts/10158053045049843" title="Stöten i Sälen postade på Facebook">
            <data>
                <context>Social</context>
                <provider>Facebook</provider>
            </data>
        </link>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-f5da770b839dc92b2bcfae54c804ae97',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/facebook-post',
                    'url' => 'https://www.facebook.com/stotenisalen/posts/10158053045049843',
                    'uri' => 'im://facebook-post/10158053045049843',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://www.facebook.com/stotenisalen/posts/10158053045049843',
                    'title' => 'Stöten i Sälen postade på Facebook',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_parse_facebook_page_object()
    {
        $xml = <<<XML
<object id="socialembed-221465e03d2a77f0836b85ee62851504" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/facebook-page" url="https://www.facebook.com/infomakerscandinavia" uri="im://facebook-page/infomakerscandinavia"/>
        <link rel="alternate" type="text/html" url="https://www.facebook.com/infomakerscandinavia" title="infomakerscandinavia">
            <data>
                <context>Social</context>
                <provider>Facebook</provider>
            </data>
        </link>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-221465e03d2a77f0836b85ee62851504',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/facebook-page',
                    'url' => 'https://www.facebook.com/infomakerscandinavia',
                    'uri' => 'im://facebook-page/infomakerscandinavia',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://www.facebook.com/infomakerscandinavia',
                    'title' => 'infomakerscandinavia',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_parse_facebook_video_object()
    {
        $xml = <<<XML
<object id="socialembed-0c2a57aa637e696ba4b2eb99c3b3e022" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/facebook-video" url="https://www.facebook.com/130004133621/videos/870816339922635/" uri="im://facebook-video/870816339922635"/>
        <link rel="alternate" type="text/html" url="https://www.facebook.com/130004133621/videos/870816339922635/" title="Naviga Content Lab postade på Facebook">
            <data>
                <context>Social</context>
                <provider>Facebook</provider>
            </data>
        </link>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-0c2a57aa637e696ba4b2eb99c3b3e022',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/facebook-video',
                    'url' => 'https://www.facebook.com/130004133621/videos/870816339922635/',
                    'uri' => 'im://facebook-video/870816339922635',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://www.facebook.com/130004133621/videos/870816339922635/',
                    'title' => 'Naviga Content Lab postade på Facebook',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_parse_twitter_object()
    {
        $xml = <<<XML
<object id="socialembed-4c4cfaef42e8cf621190fc66e8089d58" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/tweet" url="https://twitter.com/infomakernews/status/910422014271348736" uri="im://tweet/910422014271348736"/>
        <link rel="alternate" type="text/html" url="https://twitter.com/infomakernews/status/910422014271348736" title="Infomaker skrev på Twitter"/>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-4c4cfaef42e8cf621190fc66e8089d58',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/tweet',
                    'url' => 'https://twitter.com/infomakernews/status/910422014271348736',
                    'uri' => 'im://tweet/910422014271348736',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://twitter.com/infomakernews/status/910422014271348736',
                    'title' => 'Infomaker skrev på Twitter',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_parse_a_vimeo_object()
    {
        $xml = <<<XML
<object id="socialembed-6d732e68f08c1022d90c9c45d6affd3e" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/vimeo" url="https://vimeo.com/76979871" uri="im://vimeo/76979871"/>
        <link rel="alternate" type="text/html" url="https://vimeo.com/76979871" title="Vimeo Staff postade på Vimeo">
            <data>
                <context>Social</context>
                <provider>Vimeo</provider>
            </data>
        </link>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-6d732e68f08c1022d90c9c45d6affd3e',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/vimeo',
                    'url' => 'https://vimeo.com/76979871',
                    'uri' => 'im://vimeo/76979871',
                    'embedId' => '76979871'
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://vimeo.com/76979871',
                    'title' => 'Vimeo Staff postade på Vimeo',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_parse_soundcloud_object()
    {
        $xml = <<<XML
<object id="socialembed-5a61d839cd956b1d511e2272b9d62c10" type="x-im/socialembed">
    <links>
        <link rel="self" type="x-im/soundcloud" url="https://soundcloud.com/angrysausage/toby-fox-undertale" uri="im://soundcloud/angrysausage.Toby%20Fox%20-%20UNDERTALE%20Soundtrack%20-%20100%20MEGALOVANIA%20by%20angrysausage"/>
        <link rel="alternate" type="text/html" url="https://soundcloud.com/angrysausage/toby-fox-undertale" title="angrysausage postade på SoundCloud">
            <data>
                <context>Social</context>
                <provider>SoundCloud</provider>
            </data>
        </link>
    </links>
</object>
XML;
        $match = [
            'id' => 'socialembed-5a61d839cd956b1d511e2272b9d62c10',
            'type' => 'x-im/socialembed',
            'content' => [
                [
                    'rel' => 'self',
                    'type' => 'x-im/soundcloud',
                    'url' => 'https://soundcloud.com/angrysausage/toby-fox-undertale',
                    'uri' => 'im://soundcloud/angrysausage.Toby%20Fox%20-%20UNDERTALE%20Soundtrack%20-%20100%20MEGALOVANIA%20by%20angrysausage',
                ],
                [
                    'rel' => 'alternate',
                    'type' => 'text/html',
                    'url' => 'https://soundcloud.com/angrysausage/toby-fox-undertale',
                    'title' => 'angrysausage postade på SoundCloud',
                ]
            ]
        ];

        $this->validateXML($xml, $match);
    }

    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/socialembed');
    }

    private function validateXML($xml, array $matchValues)
    {
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)->shouldReturnValidItem($matchValues);
    }
}
