<?php declare(strict_types=1);

namespace Spec\Matchers;

use Infomaker\Everyware\NewsML\Item;
use PhpSpec\Exception\Example\FailureException;
use PhpSpec\Matcher\BasicMatcher;

/**
 * Class ValidItem
 * @package Spec\Matchers
 */
class ValidItem extends BasicMatcher
{
    private $names = [
        'beValidItem',
        'returnValidItem'
    ];

    protected function matches($subject, array $arguments): bool
    {
        $subjects = (array)$subject;

        if (empty($subjects)) {
            throw new FailureException('Did not return any parsed items.');
        }

        $item = $subjects[0];

        if ( ! $item instanceof Item) {
            throw new FailureException(sprintf('Failed to parse the object into a "%s".', Item::class));
        }

        if (empty($arguments)) {
            return true;
        }

        $properties = $arguments[0];
        $parsedProperties = $item->toArray();

        foreach ($properties as $name => $value) {
            $this->validateProperty($name, $value, $parsedProperties);
        }

        if ($parsedProperties !== $properties) {
            throw new FailureException(implode(PHP_EOL, [
                    'The Item did not match desired the properties.',
                    'expected ',
                    json_encode($properties, JSON_PRETTY_PRINT),
                    'but got ',
                    $item->toJson(JSON_PRETTY_PRINT)
                ])
            );
        }

        return true;
    }

    protected function getFailureException(string $name, $subject, array $arguments): FailureException
    {
        return new FailureException(sprintf(
            'Expected the subject: %s %s %sto be equal to %s %s',
            "\n",
            json_encode($subject),
            "\n",
            "\n",
            json_encode($arguments[0])
        ));
    }

    protected function getNegativeFailureException(string $name, $subject, array $arguments): FailureException
    {
        return new FailureException(sprintf(
            'Expected the subject: %s %s %sto be equal to %s %s',
            "\n",
            json_encode($subject),
            "\n",
            "\n",
            json_encode($arguments[0])
        ));
    }

    public function supports(string $name, $subject, array $arguments): bool
    {
        return in_array($name, $this->names) && count($arguments) === 1 && is_array($arguments[0]);
    }

    private function validateProperty(string $name, $value, array $properties): void
    {
        if ( ! array_key_exists($name, $properties)) {
            throw new FailureException(sprintf('The Item did not match the desired property: "%s"', $name));
        }

        if (is_array($value)) {
            foreach ($value as $key => $propertyObject) {
                $propertiesToMatch = $properties[$name][$key];
                foreach ($propertyObject as $propertyName => $propertyValue) {
                    $this->validateProperty($propertyName, $propertyValue, $propertiesToMatch);
                }
            }
        } elseif ($value !== $properties[$name]) {
            throw new FailureException(implode(PHP_EOL, [
                sprintf('The Property: "%s" does not match the desired value.', $name),
                'expected ',
                "\"{$properties[$name]}\"",
                'but got ',
                "\"{$value}\""
            ]));
        }
    }
}
